@@ -141,6 +141,7 @@ static const struct option long_options[] = {
   { "force-level",        required_argument, NULL, 0 },
   { "high-tier",                no_argument, NULL, 0 },
   { "me-steps",           required_argument, NULL, 0 },
+  { "roi-file",           required_argument, NULL, 0 },
   { "fast-residual-cost", required_argument, NULL, 0 },
   { "set-qp-in-cu",             no_argument, NULL, 0 },
   { "open-gop",                 no_argument, NULL, 0 },
@@ -167,6 +168,14 @@ static const struct option long_options[] = {
   { "fastrd-sampling",          no_argument, NULL, 0 },
   { "fastrd-accuracy-check",    no_argument, NULL, 0 },
   { "fastrd-outdir",      required_argument, NULL, 0 },
+  { "combine-intra-cus",        no_argument, NULL, 0 },
+  { "no-combine-intra-cus",     no_argument, NULL, 0 },
+  { "force-inter",              no_argument, NULL, 0 },
+  { "no-force-inter",           no_argument, NULL, 0 },
+  { "intra-chroma-search",      no_argument, NULL, 0 },
+  { "no-intra-chroma-search",   no_argument, NULL, 0 },
+  { "fast-bipred",              no_argument, NULL, 0 },
+  { "no-fast-bipred",           no_argument, NULL, 0 },
   {0, 0, 0, 0}
 };
 
@@ -500,11 +509,20 @@ void print_help(void)
     "                                   - frametile: Constrain within the tile.\n"
     "                                   - frametilemargin: Constrain even more.\n"
     "      --roi <filename>       : Use a delta QP map for region of interest.\n"
-    "                               Reads an array of delta QP values from a text\n"
-    "                               file. The file format is: width and height of\n"
-    "                               the QP delta map followed by width*height delta\n"
-    "                               QP values in raster order. The map can be of any\n"
-    "                               size and will be scaled to the video size.\n"
+    "                               Reads an array of delta QP values from a file.\n"
+    "                               Text and binary files are supported and detected\n"
+    "                               from the file extension (.txt/.bin). If a known\n"
+    "                               extension is not found, the file is treated as\n"
+    "                               a text file. The file can include one or many\n"
+    "                               ROI frames each in the following format:\n"
+    "                               width and height of the QP delta map followed\n"
+    "                               by width * height delta QP values in raster\n"
+    "                               order. In binary format, width and height are\n"
+    "                               32-bit integers whereas the delta QP values are\n"
+    "                               signed 8-bit values. The map can be of any size\n"
+    "                               and will be scaled to the video size. The file\n"
+    "                               reading will loop if end of the file is reached.\n"
+    "                               See roi.txt in the examples folder.\n"
     "      --set-qp-in-cu         : Set QP at CU level keeping pic_init_qp_minus26.\n"
     "                               in PPS and slice_qp_delta in slize header zero.\n"
     "      --(no-)erp-aqp         : Use adaptive QP for 360 degree video with\n"
@@ -536,18 +554,22 @@ void print_help(void)
     "      --(no-)signhide        : Sign hiding [disabled]\n"
     "      --(no-)smp             : Symmetric motion partition [disabled]\n"
     "      --(no-)amp             : Asymmetric motion partition [disabled]\n"
-    "      --rd <integer>         : Intra mode search complexity [0]\n"
+    "      --rd <integer>         : Mode search complexity [0]\n"
     "                                   - 0: Skip intra if inter is good enough.\n"
     "                                   - 1: Rough intra mode search with SATD.\n"
-    "                                   - 2: Refine intra mode search with SSE.\n"
-    "                                   - 3: Try all intra modes and enable intra\n"
-    "                                        chroma mode search.\n"
+    "                                   - 2: Refine mode search with SSE.\n"
+    "                                   - 3: More SSE candidates for inter and\n"
+    "                                        chroma mode search for 4x4 intra.\n"
+    "                                   - 4: Even more SSE candidates for both.\n"
+    "                                   - 5: Try all intra modes.\n"
     "      --(no-)mv-rdo          : Rate-distortion optimized motion vector costs\n"
     "                               [disabled]\n"
     "      --(no-)zero-coeff-rdo  : If a CU is set inter, check if forcing zero\n"
     "                               residual improves the RD cost. [enabled]\n"
     "      --(no-)full-intra-search : Try all intra modes during rough search.\n"
     "                               [disabled]\n"
+    "      --(no-)intra-chroma-search : Test non-derived intra chroma modes.\n"
+    "                                   [disabled]\n"
     "      --(no-)transform-skip  : Try transform skip [disabled]\n"
     "      --me <string>          : Integer motion estimation algorithm [hexbs]\n"
     "                                   - hexbs: Hexagon Based Search\n"
@@ -563,6 +585,7 @@ void print_help(void)
     "                                   - 2: + 1/2-pixel diagonal\n"
     "                                   - 3: + 1/4-pixel horizontal and vertical\n"
     "                                   - 4: + 1/4-pixel diagonal\n"
+    "      --(no-)fast-bipred     : Only perform fast bipred search. [enabled]\n"
     "      --pu-depth-inter <int>-<int> : Inter prediction units sizes [0-3]\n"
     "                                   - 0, 1, 2, 3: from 64x64 to 8x8\n"
     "                                   - Accepts a list of values separated by ','\n"
@@ -578,6 +601,16 @@ void print_help(void)
     "      --ml-pu-depth-intra    : Predict the pu-depth-intra using machine\n"
     "                                learning trees, overrides the\n"
     "                                --pu-depth-intra parameter. [disabled]\n"
+    "      --(no-)combine-intra-cus: Whether the encoder tries to code a cu\n"
+    "                                   on lower depth even when search is not\n"
+    "                                   performed on said depth. Should only\n"
+    "                                   be disabled if cus absolutely must not\n"
+    "                                   be larger than limited by the search.\n"
+    "                                   [enabled]\n"
+    "      --force-inter          : Force the encoder to use inter always.\n"
+    "                               This is mostly for debugging and is not\n"
+    "                               guaranteed to produce sensible bitstream or\n"
+    "                               work at all. [disabled]\n"
     "      --tr-depth-intra <int> : Transform split depth for intra blocks [0]\n"
     "      --(no-)bipred          : Bi-prediction [disabled]\n"
     "      --cu-split-termination <string> : CU split search termination [zero]\n"
@@ -633,8 +666,8 @@ void print_help(void)
     "                                   - u<int>: Number of tile columns of uniform\n"
     "                                             width.\n"
     "      --tiles-height-split <string>|u<int> :\n"
-    "                                   - <string>: A comma-separated list of tile row\n"
-    "                                               column pixel coordinates.\n"
+    "                                   - <string>: A comma-separated list of tile\n"
+    "                                               row column pixel coordinates.\n"
     "                                   - u<int>: Number of tile rows of uniform\n"
     "                                             height.\n"
     "      --slices <string>      : Control how slices are used.\n"
