@@ -9,15 +9,15 @@
  * 
  * * Redistributions of source code must retain the above copyright notice, this
  *   list of conditions and the following disclaimer.
- * 
+ *
  * * Redistributions in binary form must reproduce the above copyright notice, this
  *   list of conditions and the following disclaimer in the documentation and/or
  *   other materials provided with the distribution.
- * 
+ *
  * * Neither the name of the Tampere University or ITU/ISO/IEC nor the names of its
  *   contributors may be used to endorse or promote products derived from
  *   this software without specific prior written permission.
- * 
+ *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
  * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
  * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
@@ -48,43 +48,44 @@
 #include "kvz_math.h"
 #include "nal.h"
 #include "scalinglist.h"
+#include "sei.h"
 #include "tables.h"
 #include "threadqueue.h"
 #include "videoframe.h"
 #include "rate_control.h"
 
 
-static void encoder_state_write_bitstream_aud(encoder_state_t * const state)
+static void encoder_state_write_bitstream_aud(encoder_state_t *const state)
 {
-  bitstream_t * const stream = &state->stream;
+  bitstream_t *const stream = &state->stream;
   kvz_nal_write(stream, KVZ_NAL_AUD_NUT, 0, 1);
 
   uint8_t pic_type = state->frame->slicetype == KVZ_SLICE_I ? 0
-                   : state->frame->slicetype == KVZ_SLICE_P ? 1
-                   :                                       2;
+    : state->frame->slicetype == KVZ_SLICE_P ? 1
+    : 2;
   WRITE_U(stream, pic_type, 3, "pic_type");
 
   kvz_bitstream_add_rbsp_trailing_bits(stream);
 }
 
 static void encoder_state_write_bitstream_PTL(bitstream_t *stream,
-                                              encoder_state_t * const state)
+  encoder_state_t *const state)
 {
   // PTL
   // Profile Tier
   WRITE_U(stream, 0, 2, "general_profile_space");
   WRITE_U(stream, state->encoder_control->cfg.high_tier, 1, "general_tier_flag");
   // Main Profile == 1,  Main 10 profile == 2
-  WRITE_U(stream, (state->encoder_control->bitdepth == 8)?1:2, 5, "general_profile_idc");
+  WRITE_U(stream, (state->encoder_control->bitdepth == 8) ? 1 : 2, 5, "general_profile_idc");
   /* Compatibility flags should be set at general_profile_idc
    *  (so with general_profile_idc = 1, compatibility_flag[1] should be 1)
    * According to specification, when compatibility_flag[1] is set,
    *  compatibility_flag[2] should be set too.
    */
-  WRITE_U(stream, 3<<29, 32, "general_profile_compatibility_flag[]");
+  WRITE_U(stream, 3 << 29, 32, "general_profile_compatibility_flag[]");
 
   WRITE_U(stream, 1, 1, "general_progressive_source_flag");
-  WRITE_U(stream, state->encoder_control->in.source_scan_type!= 0, 1, "general_interlaced_source_flag");
+  WRITE_U(stream, state->encoder_control->in.source_scan_type != 0, 1, "general_interlaced_source_flag");
   WRITE_U(stream, 0, 1, "general_non_packed_constraint_flag");
   WRITE_U(stream, 0, 1, "general_frame_only_constraint_flag");
 
@@ -106,6 +107,25 @@ static void encoder_state_write_bitstream_PTL(bitstream_t *stream,
   // end PTL
 }
 
+static uint8_t max_required_dpb_size(const encoder_control_t * const encoder)
+{
+  int max_buffer = 1;
+  for (int g = 0; g < encoder->cfg.gop_len; ++g) {
+    int neg_refs = encoder->cfg.gop[g].ref_neg_count;
+    int pos_refs = encoder->cfg.gop[g].ref_pos_count;
+    if (neg_refs + pos_refs + 1 > max_buffer) max_buffer = neg_refs + pos_refs + 1;
+  }
+
+  if (encoder->cfg.gop_len == 0) max_buffer = encoder->cfg.ref_frames + 1;
+
+  return max_buffer;
+}
+
+static uint8_t max_num_reorder_pics(const encoder_control_t * const encoder)
+{
+  return encoder->cfg.gop_lowdelay ? 0 : MAX(encoder->cfg.gop_len - 1, 0);
+}
+
 static void encoder_state_write_bitstream_vid_parameter_set(bitstream_t* stream,
                                                             encoder_state_t * const state)
 {
@@ -125,17 +145,12 @@ static void encoder_state_write_bitstream_vid_parameter_set(bitstream_t* stream,
 
   WRITE_U(stream, 0, 1, "vps_sub_layer_ordering_info_present_flag");
 
-  if (encoder->cfg.gop_lowdelay) {
-    const int dpb = encoder->cfg.ref_frames;
-    WRITE_UE(stream, dpb - 1, "vps_max_dec_pic_buffering_minus1");
-    WRITE_UE(stream, 0, "vps_max_num_reorder_pics");
-  }
-  else {
-    // Clip to non-negative values to prevent problems with GOP=0
-    const int dpb = MIN(16, encoder->cfg.gop_len);
-    WRITE_UE(stream, MAX(dpb - 1, 0), "vps_max_dec_pic_buffering_minus1");
-    WRITE_UE(stream, MAX(encoder->cfg.gop_len - 1, 0), "vps_max_num_reorder_pics");
-  }
+  int max_buffer  = max_required_dpb_size(encoder);
+  int max_reorder = max_num_reorder_pics(encoder);
+  if (max_buffer - 1 < max_reorder) max_buffer = max_reorder + 1;
+  WRITE_UE(stream, max_buffer - 1, "vps_max_dec_pic_buffering_minus1");
+  WRITE_UE(stream, max_reorder, "vps_max_num_reorder_pics");
+
   WRITE_UE(stream, 0, "vps_max_latency_increase");
 
   WRITE_U(stream, 0, 6, "vps_max_nuh_reserved_zero_layer_id");
@@ -402,16 +417,12 @@ static void encoder_state_write_bitstream_seq_parameter_set(bitstream_t* stream,
   WRITE_U(stream, 0, 1, "sps_sub_layer_ordering_info_present_flag");
 
   //for each layer
-  if (encoder->cfg.gop_lowdelay) {
-    const int dpb = encoder->cfg.ref_frames;
-    WRITE_UE(stream, dpb - 1, "sps_max_dec_pic_buffering_minus1");
-    WRITE_UE(stream, 0, "sps_max_num_reorder_pics");
-  } else {
-    // Clip to non-negative values to prevent problems with GOP=0
-    const int dpb = MIN(16, encoder->cfg.gop_len);
-    WRITE_UE(stream, MAX(dpb - 1, 0), "sps_max_dec_pic_buffering_minus1");
-    WRITE_UE(stream, MAX(encoder->cfg.gop_len - 1, 0), "sps_max_num_reorder_pics");
-  }
+  int max_buffer  = max_required_dpb_size(encoder);
+  int max_reorder = max_num_reorder_pics(encoder);
+  if (max_buffer - 1 < max_reorder) max_buffer = max_reorder + 1;
+  WRITE_UE(stream, max_buffer - 1, "sps_max_dec_pic_buffering_minus1");
+  WRITE_UE(stream, max_reorder, "sps_max_num_reorder_pics");
+
   WRITE_UE(stream, 0, "sps_max_latency_increase_plus1");
   //end for
 
@@ -493,10 +504,10 @@ static void encoder_state_write_bitstream_pic_parameter_set(bitstream_t* stream,
   WRITE_U(stream, 0, 1, "constrained_intra_pred_flag");
   WRITE_U(stream, encoder->cfg.trskip_enable, 1, "transform_skip_enabled_flag");
 
-  if (encoder->max_qp_delta_depth >= 0) {
+  if (state->frame->max_qp_delta_depth >= 0) {
     // Use separate QP for each LCU when rate control is enabled.
     WRITE_U(stream, 1, 1, "cu_qp_delta_enabled_flag");
-    WRITE_UE(stream, encoder->max_qp_delta_depth, "diff_cu_qp_delta_depth");
+    WRITE_UE(stream, state->frame->max_qp_delta_depth, "diff_cu_qp_delta_depth");
   } else {
     WRITE_U(stream, 0, 1, "cu_qp_delta_enabled_flag");
   }
@@ -560,22 +571,47 @@ static void encoder_state_write_bitstream_pic_parameter_set(bitstream_t* stream,
   kvz_bitstream_add_rbsp_trailing_bits(stream);
 }
 
+static void sei_write_payload_type(bitstream_t *stream, const int payloadType)
+{
+  int i;
+  for (i = 0; i <= payloadType - 255; i += 255) {
+    WRITE_U(stream, FF_BYTE, 8, "ff_byte");
+  }
+  WRITE_U(stream, payloadType - i, 8, "last_payload_type_byte");
+}
+
+static void sei_write_payload_size(bitstream_t *stream, const int payloadSize)
+{
+  int i;
+  for (i = 0; i <= payloadSize - 255; i += 255) {
+    WRITE_U(stream, FF_BYTE, 8, "ff_byte");
+  }
+  WRITE_U(stream, payloadSize - i, 8, "last_payload_size_byte");
+}
+
+static void sei_write_user_defined_unregistered(bitstream_t *stream, const uint8_t * const uuid, const uint8_t * const user_data_payload_byte, const int length)
+{
+  int i;
+  sei_write_payload_type(stream, SEI_PAYLOAD_TYPE_USER_DATA_UNREGISTERED);
+  sei_write_payload_size(stream, (sizeof encoder_info_uuid) + length);
+  for (i = 0; i < 16; i++) {
+    WRITE_U(stream, uuid[i], 8, "uuid_iso_iec_11578");
+  }
+  for (i = 0; i < length; i++) {
+    WRITE_U(stream, user_data_payload_byte[i], 8, "user_data_payload_byte");
+  }
+  kvz_bitstream_align(stream);
+}
+
 static void encoder_state_write_bitstream_prefix_sei_version(encoder_state_t * const state)
 {
 #define STR_BUF_LEN 1000
   bitstream_t * const stream = &state->stream;
-  int i, length;
+  int length;
   char buf[STR_BUF_LEN] = { 0 };
-  char *s = buf + 16;
+  char *s = buf;
   const kvz_config * const cfg = &state->encoder_control->cfg;
 
-  // random uuid_iso_iec_11578 generated with www.famkruithof.net/uuid/uuidgen
-  static const uint8_t uuid[16] = {
-    0x32, 0xfe, 0x46, 0x6c, 0x98, 0x41, 0x42, 0x69,
-    0xae, 0x35, 0x6a, 0x91, 0x54, 0x9e, 0xf3, 0xf1
-  };
-  memcpy(buf, uuid, 16);
-
   // user_data_payload_byte
   s += sprintf(s, "Kvazaar HEVC Encoder v. " VERSION_STRING " - "
                   "Copyleft 2012-2015 - http://ultravideo.cs.tut.fi/ - options:");
@@ -593,19 +629,7 @@ static void encoder_state_write_bitstream_prefix_sei_version(encoder_state_t * c
   // to increase the buf len. Divide by 2 for margin.
   assert(length < STR_BUF_LEN / 2);
 
-  // payloadType = 5 -> user_data_unregistered
-  WRITE_U(stream, 5, 8, "last_payload_type_byte");
-
-  // payloadSize
-  for (i = 0; i <= length - 255; i += 255)
-    WRITE_U(stream, 255, 8, "ff_byte");
-  WRITE_U(stream, length - i, 8, "last_payload_size_byte");
-
-  for (i = 0; i < length; i++)
-    WRITE_U(stream, ((uint8_t *)buf)[i], 8, "sei_payload");
-
-  // The bitstream is already aligned, but align it anyway.
-  kvz_bitstream_align(stream);
+  sei_write_user_defined_unregistered(stream, encoder_info_uuid, (uint8_t*)buf, length);
 
 #undef STR_BUF_LEN
 }
@@ -673,8 +697,8 @@ static void encoder_state_write_picture_timing_sei_message(encoder_state_t * con
       break;
     }
 
-    WRITE_U(stream, 1, 8, "last_payload_type_byte"); //pic_timing
-    WRITE_U(stream, 1, 8, "last_payload_size_byte");
+    sei_write_payload_type(stream, SEI_PAYLOAD_TYPE_PIC_TIMING);
+    sei_write_payload_size(stream, 1);
     WRITE_U(stream, pic_struct, 4, "pic_struct");
     WRITE_U(stream, source_scan_type, 2, "source_scan_type");
     WRITE_U(stream, 0, 1, "duplicate_flag");
@@ -928,7 +952,6 @@ void kvz_encoder_state_write_bitstream_slice_header(
   }
 }
 
-
 /**
  * \brief Add a checksum SEI message to the bitstream.
  * \param encoder The encoder.
@@ -942,7 +965,7 @@ static void add_checksum(encoder_state_t * const state)
 
   kvz_nal_write(stream, KVZ_NAL_SUFFIX_SEI_NUT, 0, 0);
 
-  WRITE_U(stream, 132, 8, "sei_type");
+  sei_write_payload_type(stream, SEI_PAYLOAD_TYPE_DECODED_PICTURE_HASH);
 
   int num_colors = (state->encoder_control->chroma_format == KVZ_CSP_400 ? 1 : 3);
 
@@ -951,7 +974,7 @@ static void add_checksum(encoder_state_t * const state)
   case KVZ_HASH_CHECKSUM:
     kvz_image_checksum(frame->rec, checksum, state->encoder_control->bitdepth);
 
-    WRITE_U(stream, 1 + num_colors * 4, 8, "size");
+    sei_write_payload_size(stream, 1 + num_colors * 4);
     WRITE_U(stream, 2, 8, "hash_type");  // 2 = checksum
 
     for (int i = 0; i < num_colors; ++i) {
@@ -967,7 +990,7 @@ static void add_checksum(encoder_state_t * const state)
   case KVZ_HASH_MD5:
     kvz_image_md5(frame->rec, checksum, state->encoder_control->bitdepth);
 
-    WRITE_U(stream, 1 + num_colors * 16, 8, "size");
+    sei_write_payload_size(stream, 1 + num_colors * 16);
     WRITE_U(stream, 0, 8, "hash_type");  // 0 = md5
 
     for (int i = 0; i < num_colors; ++i) {
